/*
 * Copyright (c) 1992-1998 Michael A. Cooper.
 * This software may be freely used and distributed provided it is not
 * sold for profit or used in part or in whole for commercial gain
 * without prior written agreement, and the author is credited
 * appropriately.
 */

/*
 * $Id: version.h,v 6.33 1998/11/10 04:25:51 mcooper Exp $
 */

/*
 * The rdist protocol version number.  This should be changed
 * whenever the protocol changes.
 */
#define VERSION			6

/*
 * Distribution version.  The major distribution number should match
 * the protocol version number (VERSION) defined above.
 */
#define DISTVERSION		"6.1"

/*
 * Patch level
 */
#define PATCHLEVEL 		5

/*
 * Distribution status
 */
#define DISTSTATUS		"RELEASE"
